@@ -21,8 +21,10 @@
 #include <config.h>
 
 #include <boost/bind.hpp>
+#include <boost/checked_delete.hpp>
 
 #include "sharp/map.hpp"
+#include "sharp/directory.hpp"
 #include "sharp/dynamicmodule.hpp"
 
 #include "addinmanager.hpp"
@@ -32,6 +34,29 @@
 #include "notebooks/notebookapplicationaddin.hpp"
 #include "notebooks/notebooknoteaddin.hpp"
 
+
+#if 1
+
+// HACK to make sure we link the modules needed only for addins
+#include "base/inifile.hpp"
+#include "sharp/fileinfo.hpp"
+#include "sharp/streamwriter.hpp"
+#include "sharp/xsltransform.hpp"
+#include "sharp/xsltargumentlist.hpp"
+
+bool links[] = {
+  base::IniFileLink_,
+  sharp::FileInfoLink_,
+  sharp::StreamWriterLink_,
+  sharp::XslTransformLink_,
+  sharp::XsltArgumentListLink_,
+  false
+};
+
+
+#endif
+
+
 namespace gnote {
 
 #define REGISTER_BUILTIN_NOTE_ADDIN(klass) \
@@ -46,6 +71,7 @@ namespace gnote {
   AddinManager::AddinManager(const std::string & conf_dir)
     : m_gnote_conf_dir(conf_dir)
   {
+    m_addins_prefs_dir = Glib::build_filename(conf_dir, "addins");
     initialize_sharp_addins();
   }
 
@@ -66,6 +92,15 @@ namespace gnote {
 
   void AddinManager::initialize_sharp_addins()
   {
+
+    try {
+      if(!sharp::directory_exists(m_addins_prefs_dir)) {
+        sharp::directory_create(m_addins_prefs_dir);
+      }
+    }
+    catch(...) {
+
+    }
     // get the factory
 
     REGISTER_BUILTIN_NOTE_ADDIN(NoteRenameWatcher);
@@ -103,7 +138,12 @@ namespace gnote {
         AddinPreferenceFactoryBase * factory = dynamic_cast<AddinPreferenceFactoryBase*>((*f)());
         m_addin_prefs.insert(std::make_pair((*iter)->id(), factory));
       }
-      
+
+      f = dmod->query_interface(ImportAddin::IFACE_NAME);
+      if(f) {
+        ImportAddin * factory = dynamic_cast<ImportAddin*>((*f)());
+        m_import_addins.insert(std::make_pair((*iter)->id(), factory));
+      }
     }
   }
 
@@ -142,6 +182,13 @@ namespace gnote {
     sharp::map_get_values(m_pref_tab_addins, l);
   }
 
+
+  void AddinManager::get_import_addins(std::list<ImportAddin*> & l) const
+  {
+    sharp::map_get_values(m_import_addins, l);
+  }
+
+
   Gtk::Widget * AddinManager::create_addin_preference_widget(const std::string & id)
   {
     IdAddinPrefsMap::const_iterator iter = m_addin_prefs.find(id);
