@@ -32,9 +32,7 @@
 #include <boost/filesystem/path.hpp>
 #include <boost/algorithm/string/find.hpp>
 
-#include <libxml++/parsers/textreader.h>
-#include <libxml++/parsers/domparser.h>
-#include <libxml++/nodes/textnode.h>
+#include <libxml/parser.h>
 
 #include <glibmm/i18n.h>
 #include <gtkmm/button.h>
@@ -51,7 +49,9 @@
 #include "sharp/files.hpp"
 #include "sharp/map.hpp"
 #include "sharp/string.hpp"
+#include "sharp/xml.hpp"
 #include "sharp/xmlconvert.hpp"
+#include "sharp/xmlreader.hpp"
 #include "sharp/xmlwriter.hpp"
 
 
@@ -667,17 +667,30 @@ namespace gnote {
 
   void Note::remove_tag(Tag & tag)
   {
+    std::string tag_name = tag.normalized_name();
     NoteData::TagMap & thetags(m_data.data().tags());
-    NoteData::TagMap::iterator iter = thetags.find(tag.normalized_name());
-    if (iter == thetags.end())
-      return;
+    NoteData::TagMap::iterator iter;
+
+    // if we are deleting the note, no need to check for the tag, we 
+    // know it is there.
+    if(!m_is_deleting) {
+      iter = thetags.find(tag_name);
+      if (iter == thetags.end())  {
+        return;
+      }
+    }
 
     m_signal_tag_removing(*this, tag);
 
-    thetags.erase(iter);
+    // don't erase the tag if we are deleting the note. 
+    // This will invalidate the iterator.
+    // see bug 579839.
+    if(!m_is_deleting) {
+      thetags.erase(iter);
+    }
     tag.remove_note(*this);
 
-    m_signal_tag_removed(shared_from_this(), tag.normalized_name());
+    m_signal_tag_removed(shared_from_this(), tag_name);
 
     DBG_OUT("Tag removed, queueing save");
     queue_save(OTHER_DATA_CHANGED);
@@ -796,16 +809,15 @@ namespace gnote {
     // were to throw an XmlException in the middle of processing,
     // a note could be damaged.  Therefore, we check for parseability
     // ahead of time, and throw early.
-    {
-      xmlpp::DomParser parser;
-      parser.parse_memory(foreignNoteXml);
-      if(!parser) {
-        throw sharp::Exception("invalid XML in foreignNoteXml");
-      }
-    }
+    xmlDocPtr doc = xmlParseDoc((const xmlChar *)foreignNoteXml.c_str());
 
-    xmlpp::TextReader xml((const unsigned char*)foreignNoteXml.c_str(), 
-                          foreignNoteXml.size());
+    if(!doc) {
+      throw sharp::Exception("invalid XML in foreignNoteXml");
+    }
+    xmlFreeDoc(doc);
+
+    sharp::XmlReader xml;
+    xml.load_buffer(foreignNoteXml);
 
     // Remove tags now, since a note with no tags has
     // no "tags" element in the XML
@@ -814,11 +826,11 @@ namespace gnote {
         iter != tag_list.end(); ++iter) {
       remove_tag(*iter);
     }
-    Glib::ustring name;
+    std::string name;
 
     while (xml.read()) {
       switch (xml.get_node_type()) {
-      case xmlpp::TextReader::Element:
+      case XML_READER_TYPE_ELEMENT:
         name = xml.get_name();
         if (name == "title") {
           set_title(xml.read_string());
@@ -839,17 +851,16 @@ namespace gnote {
             sharp::XmlConvert::to_date_time(xml.read_string ());
         }
         else if(name == "tags") {
-          xmlpp::DomParser parser;
-          parser.parse_memory(xml.read_outer_xml());
-          if(parser) {
-            const xmlpp::Document * doc2 = parser.get_document();
+          xmlDocPtr doc2 = xmlParseDoc((const xmlChar*)xml.read_outer_xml().c_str());
+          if(doc2) {
             std::list<std::string> tag_strings;
-            parse_tags (doc2->get_root_node(), tag_strings);
+            parse_tags (doc2->children, tag_strings);
             for(std::list<std::string>::const_iterator iter = tag_strings.begin();
                 iter != tag_strings.end(); ++iter) {
               Tag::Ptr tag = TagManager::obj().get_or_create_tag(*iter);
               add_tag(tag);
             }
+            xmlFreeDoc(doc2);
           }
           else {
             DBG_OUT("loading tag subtree failed");
@@ -873,22 +884,23 @@ namespace gnote {
   }
 
 
-  void Note::parse_tags(const xmlpp::Node *tagnodes, std::list<std::string> & tags)
+  void Note::parse_tags(const xmlNodePtr tagnodes, std::list<std::string> & tags)
   {
-    xmlpp::NodeSet nodes = tagnodes->find("//*");
-    for(xmlpp::NodeSet::const_iterator iter = nodes.begin();
+    sharp::XmlNodeSet nodes = sharp::xml_node_xpath_find(tagnodes, "//*");
+    
+    if(nodes.empty()) {
+      return;
+    }
+    for(sharp::XmlNodeSet::const_iterator iter = nodes.begin();
         iter != nodes.end(); ++iter) {
-      const xmlpp::Node * node = *iter;
-      if(node->get_name() != "tag") {
-        continue;
-      }
-      const xmlpp::Element * content = dynamic_cast<const xmlpp::Element*>(node);
-      if(content) {
-        const xmlpp::TextNode * textnode = content->get_child_text();
-        if(textnode) {
-          std::string tag = textnode->get_content();
-          DBG_OUT("found tag %s", tag.c_str());
-          tags.push_back(tag);
+
+      const xmlNodePtr node = *iter;
+      if(xmlStrEqual(node->name, (const xmlChar*)"tag") && (node->type == XML_ELEMENT_NODE)) {
+        xmlChar * content = xmlNodeGetContent(node);
+        if(content) {
+          DBG_OUT("found tag %s", content);
+          tags.push_back((const char*)content);
+          xmlFree(content);
         }
       }
     }
@@ -943,7 +955,7 @@ namespace gnote {
       // NOTE: Sharing the same TagTable means
       // that formatting is duplicated between
       // buffers.
-      m_tag_table = NoteTagTable::Ptr(&NoteTagTable::instance());
+      m_tag_table = NoteTagTable::instance();
     }
     return m_tag_table;
   }
@@ -952,7 +964,7 @@ namespace gnote {
   {
     if(!m_buffer) {
       DBG_OUT("Creating buffer for %s", m_data.data().title().c_str());
-      m_buffer = Glib::RefPtr<NoteBuffer>(new NoteBuffer(get_tag_table(), *this));
+      m_buffer = NoteBuffer::create(get_tag_table(), *this);
       m_data.set_buffer(m_buffer);
 
       m_buffer->signal_changed().connect(
@@ -1075,13 +1087,13 @@ namespace gnote {
     NoteData *note = new NoteData(uri);
     std::string version;
 
-    xmlpp::TextReader xml(read_file);
+    sharp::XmlReader xml(read_file);
 
-    Glib::ustring name;
+    std::string name;
 
     while (xml.read ()) {
       switch (xml.get_node_type()) {
-      case xmlpp::TextReader::Element:
+      case XML_READER_TYPE_ELEMENT:
         name = xml.get_name();
         
         if(name == "note") {
@@ -1123,17 +1135,17 @@ namespace gnote {
           note->y() = boost::lexical_cast<int>(xml.read_string());
         }
         else if(name == "tags") {
-          xmlpp::DomParser parser;
-          parser.parse_memory(xml.read_outer_xml());
-          if(parser) {
-            const xmlpp::Document * doc2 = parser.get_document();
+          xmlDocPtr doc2 = xmlParseDoc((const xmlChar*)xml.read_outer_xml().c_str());
+
+          if(doc2) {
             std::list<std::string> tag_strings;
-            Note::parse_tags(doc2->get_root_node(), tag_strings);
+            Note::parse_tags(doc2->children, tag_strings);
             for(std::list<std::string>::const_iterator iter = tag_strings.begin();
                 iter != tag_strings.end(); ++iter) {
               Tag::Ptr tag = TagManager::obj().get_or_create_tag(*iter);
               note->tags()[tag->normalized_name()] = tag;
             }
+            xmlFreeDoc(doc2);
           }
           else {
             DBG_OUT("loading tag subtree failed");
@@ -1153,7 +1165,7 @@ namespace gnote {
     if (version != NoteArchiver::CURRENT_VERSION) {
       // Note has old format, so rewrite it.  No need
       // to reread, since we are not adding anything.
-      DBG_OUT("Updating note XML to newest format...");
+      DBG_OUT("Updating note XML from %s to newest format...", version.c_str());
       NoteArchiver::write (read_file, *note);
     }
     return note;
@@ -1320,11 +1332,13 @@ namespace gnote {
   std::string NoteArchiver::get_title_from_note_xml(const std::string & noteXml) const
   {
     if (!noteXml.empty()) {
-      xmlpp::TextReader xml((const unsigned char*)noteXml.c_str(), noteXml.size());
+      sharp::XmlReader xml;
+
+      xml.load_buffer(noteXml);
 
       while (xml.read ()) {
         switch (xml.get_node_type()) {
-        case xmlpp::TextReader::Element:
+        case XML_READER_TYPE_ELEMENT:
           if (xml.get_name() == "title") {
             return xml.read_string ();
           }
